@@ -1,7 +1,7 @@
 package Complete::File;
 
-our $DATE = '2021-02-02'; # DATE
-our $VERSION = '0.442'; # VERSION
+our $DATE = '2021-02-08'; # DATE
+our $VERSION = '0.443'; # VERSION
 
 use 5.010001;
 use strict;
@@ -108,6 +108,19 @@ _
             schema => 'bool*',
             cmdline_aliases => {r=>{}},
         },
+        recurse_matching => {
+            schema => ['str*', in=>['level-by-level', 'all-at-once']],
+            default => 'level-by-level',
+        },
+        exclude_leaf => {
+            schema => 'bool*',
+        },
+        exclude_dir => {
+            schema => 'bool*',
+        },
+    },
+    args_rels => {
+        dep_all => [recurse_matching => ['recurse']],
     },
     result_naked => 1,
     result => {
@@ -121,7 +134,6 @@ sub complete_file {
 
     my %args   = @_;
     my $word   = $args{word} // "";
-    my $recurse = $args{recurse};
     my $handle_tilde = $args{handle_tilde} // 1;
     my $allow_dot   = $args{allow_dot} // 1;
 
@@ -266,7 +278,10 @@ sub complete_file {
         filter_func => $final_filter,
         starting_path => $starting_path,
         result_prefix => $result_prefix,
-        recurse => $recurse,
+        recurse => $args{recurse},
+        recurse_matching => $args{recurse_matching},
+        exclude_leaf => $args{exclude_leaf},
+        exclude_nonleaf => $args{exclude_nonleaf} // $args{exclude_dir},
     );
 
     # XXX why doesn't Complete::Path return hash answer with path_sep? we add
@@ -307,7 +322,7 @@ Complete::File - Completion routines related to files
 
 =head1 VERSION
 
-This document describes version 0.442 of Complete::File (from Perl distribution Complete-File), released on 2021-02-02.
+This document describes version 0.443 of Complete::File (from Perl distribution Complete-File), released on 2021-02-08.
 
 =head1 DESCRIPTION
 
@@ -335,6 +350,8 @@ If turned off, will not allow "." or ".." in path.
 This is most useful when combined with C<starting_path> option to prevent user
 going up/outside the starting path.
 
+=item * B<exclude_leaf> => I<bool>
+
 =item * B<filter> => I<str|code>
 
 Only return items matching this filter.
@@ -356,6 +373,8 @@ included.
 
 =item * B<recurse> => I<bool>
 
+=item * B<recurse_matching> => I<str> (default: "level-by-level")
+
 =item * B<starting_path> => I<str> (default: ".")
 
 =item * B<word>* => I<str> (default: "")
@@ -392,10 +411,7 @@ going up/outside the starting path.
 
 =item * B<exclude_dir> => I<bool>
 
-This is also an alternative to specifying full C<filter>. Set this to true if you
-do not want directories.
-
-If you only want directories, take a look at C<complete_dir()>.
+=item * B<exclude_leaf> => I<bool>
 
 =item * B<file_ext_filter> => I<re|array[str]>
 
@@ -432,6 +448,8 @@ included.
 
 =item * B<recurse> => I<bool>
 
+=item * B<recurse_matching> => I<str> (default: "level-by-level")
+
 =item * B<starting_path> => I<str> (default: ".")
 
 =item * B<word>* => I<str> (default: "")
